package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.qms.platform.dao.entity.*;
import com.abi.qms.platform.dao.enums.FactoryTypeEnum;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.vo.result.QueryOrderFormDetailVO;
import com.abi.qms.platform.dao.vo.result.QueryOrderFormLimitVO;
import com.abi.qms.platform.dao.vo.result.ReservationOrderQueryVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.FactoryService;
import com.abi.qms.platform.service.MaterialService;
import com.abi.qms.platform.service.QrPackageBookingOrderService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author ludashi
 * @date 2021年06月04日 15:04
 */
@Service
public class QrPackageBookingOrderServiceImpl implements QrPackageBookingOrderService {

    @Autowired
    private QrPackageBookingOrderMapper qrPackageBookingOrderMapper;

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    @Autowired
    private QrPackageMapper qrPackageMapper;

    @Autowired
    private QrPackageSapOrderItemMapper qrPackageSapOrderItemMapper;

    @Autowired
    private MaterialService materialService;

    @Autowired
    private BaseMaterialMapper baseMaterialMapper;

    @Autowired
    private FactoryService factoryService;
    /**
     * 新增修改预订单
     * @author ludashi
     * @date 2021/6/4 18:50
     * @param req
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveOrderForm(SaveOrderFormReq req) {
        QueryWrapper<QrPackageBookingOrder> qrFormatQw = new QueryWrapper<>();
        QrPackageBookingOrder qurey = null;
        if (null != req.getId()) {
            qurey = qrPackageBookingOrderMapper.selectById(req.getId());
            AssertUtil.isNull(qurey,"预订单ID不存在");
        } else {
            qurey = new QrPackageBookingOrder();
        }
        qurey.setBookingOrder(req.getBookingOrder());
        qurey.setMaterialName(req.getMaterialName());
        qurey.setPurchaseCount(req.getPurchaseCount());
        qurey.setQrType(req.getQrType());
        qurey.setFactoryCoverId(req.getFactoryCoverId());
        qurey.setDescription(req.getDescription());
        if (null != req.getId()) {
            //修改
            qurey.setBookingOrder(null);
            qrPackageBookingOrderMapper.updateById(qurey);
        } else {
            qrFormatQw.eq("booking_order", req.getBookingOrder());
            qrFormatQw.eq("is_delete", 0);
            Integer count = qrPackageBookingOrderMapper.selectCount(qrFormatQw);
            if (!count.equals(0)) {
                throw new BusinessException("预订单号已存在");
            }
            //新增
            qrPackageBookingOrderMapper.insert(qurey);
        }
    }

    /**
     * 查看预订单详情
     * @author ludashi
     * @date 2021/6/4 18:50
     * @param queryOrderFormDetailReq
     * @return com.abi.qms.platform.dto.res.QueryOrderFormDetailRes
     */
    @Override
    public QueryOrderFormDetailRes queryOrderFormDetail(QueryOrderFormDetailReq queryOrderFormDetailReq) {
        //查询预订单
        QueryWrapper<QrPackageBookingOrder> qrFormatQw = new QueryWrapper<>();
        qrFormatQw.eq("booking_order",queryOrderFormDetailReq.getBookingOrder());
        qrFormatQw.eq("is_delete",0);
        QrPackageBookingOrder qrPackageBookingOrder = qrPackageBookingOrderMapper.selectOne(qrFormatQw);
        //查询包材厂
        BaseFactory baseFactory = baseFactoryMapper.selectById(qrPackageBookingOrder.getFactoryCoverId());
        //关联码包信息
        List<QueryOrderFormDetailVO> queryQueryOrderFormDetailVO= qrPackageBookingOrderMapper.queryOrderFormDetail(queryOrderFormDetailReq);

        // 封装出参
        QueryOrderFormDetailRes orderFormDetailRes = PojoConverterUtils.copy(qrPackageBookingOrder, QueryOrderFormDetailRes.class);
        if(!ObjectUtils.isEmpty(queryQueryOrderFormDetailVO)){
            List<QueryOrderFormDetailRes.QueryPackageDetailBean> queryOrderFormDetailBeans = PojoConverterUtils.copyList(queryQueryOrderFormDetailVO, QueryOrderFormDetailRes.QueryPackageDetailBean.class);
            orderFormDetailRes.setQueryPackageDetailBean(queryOrderFormDetailBeans);
        }
        if(!ObjectUtils.isEmpty(baseFactory)){
            orderFormDetailRes.setFactoryCoverName(baseFactory.getFactoryName());
        }

        return orderFormDetailRes;
    }

    /**
     * 查看预订单查询分页
     * @author ludashi
     * @date 2021/6/7 9:45
     * @param queryOrderFormLimitReq
     */
    @Override
    public QueryOrderFormLimitRes queryOrderFormLimit(QueryOrderFormLimitReq queryOrderFormLimitReq) {
       IPage<QueryOrderFormLimitVO> iPage = qrPackageBookingOrderMapper.queryOrderFormLimit(PageUtil.createPage(queryOrderFormLimitReq), queryOrderFormLimitReq);
       List<QueryOrderFormLimitVO> records = iPage.getRecords();
       if(ObjectUtils.isEmpty(records)){
           new BusinessException("查询列表为空");
       }
       QueryOrderFormLimitRes res= new QueryOrderFormLimitRes();
       PageUtil.copyPageInfo(res, iPage);
       List<QueryOrderFormLimitRes.QueryOrderFormLimitBean> queryOrderFormLimitBeans = PojoConverterUtils.copyList(records, QueryOrderFormLimitRes.QueryOrderFormLimitBean.class);
       res.setQueryOrderFormLimitBean(queryOrderFormLimitBeans);
       return res;
    }

    /**
     * 关联sap单号
     * @author ludashi
     * @date 2021/6/7 11:45
     * @param associateSapOrderReq
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void associateSapOrder(AssociateSapOrderReq associateSapOrderReq) {
        //查询码包信息
        QueryWrapper<QrPackage> packageQuery = new QueryWrapper<>();
        packageQuery.eq("booking_order",associateSapOrderReq.getBookingOrder());
        QrPackage qrPackage = qrPackageMapper.selectOne(packageQuery);
        if(ObjectUtils.isEmpty(qrPackage)){
            throw new BusinessException("该笔预订单未关联码包，不允许直接关联SAP订单号");
        }
        //1根据物料id查询物料code保存
        BaseMaterial baseMaterials = baseMaterialMapper.selectById(associateSapOrderReq.getMaterialCode());
        BaseMaterial materialByCode = materialService.getMaterialByCode(baseMaterials.getMaterialCode());
        AssertUtil.isNull(materialByCode,"该SAP订单号内未找到需要打码的物料，请维护物料数据后重试");
        //关联SAP订单 校验
        QueryWrapper<QrPackageSapOrderItem> qrPackageSapOrderItem = new QueryWrapper<>();
        qrPackageSapOrderItem.eq("sap_order_no",associateSapOrderReq.getSapOrder());
        qrPackageSapOrderItem.eq("sap_material_id",baseMaterials.getMaterialCode());
        QrPackageSapOrderItem queryQrPackageSapOrderItem = qrPackageSapOrderItemMapper.selectOne(qrPackageSapOrderItem);
        if(ObjectUtils.isEmpty(queryQrPackageSapOrderItem)){
            new BusinessException("该SAP订单号内未找到需要打码的物料，请维护物料数据后重试");
        }
        QrPackageBookingOrder qrPackageBookingOrder = qrPackageBookingOrderMapper.selectById(associateSapOrderReq.getId());
        //查询包材厂Code
        BaseFactory factory = baseFactoryMapper.selectById(qrPackageBookingOrder.getFactoryCoverId());
        AssertUtil.isNull(factory,"包材厂信息不存在");
        //判断是否和sap订单信息一致
        if(!queryQrPackageSapOrderItem.getSapCoverFactoryId().equals(factory.getFactoryCode())){
            new BusinessException("关联的SAP订单内包材厂信息与原预订单不符，请确认后重试。");
        }
        //  物料类型 3-罐子 4-瓶盖 5-纸板箱
        if(!materialByCode.getMaterialType().equals(baseMaterials.getMaterialType())){
            new BusinessException("关联的SAP订单内物料类型与原预订单码类型不符，请确认后重试。");
        }
        QueryWrapper<QrPackage> packageQueryList = new QueryWrapper<>();
        packageQueryList.eq("sap_order_no",associateSapOrderReq.getSapOrder());
        packageQueryList.eq("material_code",baseMaterials.getMaterialCode());
        Integer integer = qrPackageMapper.selectCount(packageQueryList);
        if (integer>0) {
            throw new BusinessException("该SAP订单和物料已存在，不允许重复！");
        }
        //1修改预订单名称
        qrPackageBookingOrder.setMaterialName(baseMaterials.getMaterialName());
        qrPackageBookingOrderMapper.updateById(qrPackageBookingOrder);
        //2修改码包
        qrPackage.setSapOrderNo(associateSapOrderReq.getSapOrder());
        qrPackage.setMaterialId(baseMaterials.getId());
        qrPackage.setMaterialCode(baseMaterials.getMaterialCode());
        qrPackageMapper.updateById(qrPackage);
    }

    /**
     * 预订单下拉框选项
     * @author ludashi
     * @date 2021/6/7 17:56
     */
    @Override
    public ReservationOrderQueryRes reservationOrderQuery() {
        List<ReservationOrderQueryVO> reservationOrderQueryVOS = qrPackageBookingOrderMapper.reservationOrderQuery();
        //封装出参
        ReservationOrderQueryRes res= new ReservationOrderQueryRes();
        List<ReservationOrderQueryRes.ReservationOrderQueryBean> reservationOrderQueryBeans = PojoConverterUtils.copyList(reservationOrderQueryVOS, ReservationOrderQueryRes.ReservationOrderQueryBean.class);
        res.setReservationOrderQueryBean(reservationOrderQueryBeans);
        return res;
    }


    /**
     * 根据预订单ID删除
     * @author lu
     * @date 2021/6/8 19:07
     * @param deleteQrPackageBookingOrderReq  预订单ID
     */
    @Override
    public void deleteQrPackageBookingOrder(DeleteQrPackageBookingOrderReq deleteQrPackageBookingOrderReq) {
        QrPackageBookingOrder qrPackageBookingOrder = qrPackageBookingOrderMapper.selectById(deleteQrPackageBookingOrderReq.getId());
        AssertUtil.isNull(qrPackageBookingOrder,"预订单ID不存在");
        qrPackageBookingOrder.setIsDelete(1);
        qrPackageBookingOrderMapper.updateById(qrPackageBookingOrder);
    }

    /**
     * 获取预订单信息
     * @param bookingOrder
     * @return
     */
    @Override
    public QrPackageBookingOrder getBookingOrder(String bookingOrder) {
        QueryWrapper<QrPackageBookingOrder> bookingOrderQw = new QueryWrapper<>();
        bookingOrderQw.eq("booking_order",bookingOrder);
        bookingOrderQw.eq("is_delete",0);
        QrPackageBookingOrder qrPackageBookingOrder = qrPackageBookingOrderMapper.selectOne(bookingOrderQw);
        return qrPackageBookingOrder;
    }
}
